@@ -2,18 +2,22 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 import re
-from google.cloud import bigquery
+
 from google.api_core.exceptions import Conflict
+from google.cloud import bigquery
 
 
 def sanitize_table_name_for_bq(table_name):
-    of_good_character_but_possibly_verbose = re.sub(r'[^a-zA-Z_0-9]', '_', table_name)
+    of_good_character_but_possibly_verbose = re.sub(r"[^a-zA-Z_0-9]", "_", table_name)
 
     if len(of_good_character_but_possibly_verbose) <= 1024:
         return of_good_character_but_possibly_verbose
 
-    return of_good_character_but_possibly_verbose[:500] + '___' \
+    return (
+        of_good_character_but_possibly_verbose[:500]
+        + "___"
         + of_good_character_but_possibly_verbose[-500:]
+    )
 
 
 class BigQueryContext:
@@ -26,17 +30,16 @@ class BigQueryContext:
 
     .. _BigQuery dataset id: https://cloud.google.com/bigquery/docs/datasets
     """
-    def __init__(self, dataset_id, project_id='moz-fx-data-bq-data-science'):
+
+    def __init__(self, dataset_id, project_id="moz-fx-data-bq-data-science"):
         self.dataset_id = dataset_id
         self.project_id = project_id
         self.client = bigquery.Client(project=project_id)
 
-    def run_query(self, sql, results_table=None):
+    def run_query_and_fetch(self, sql, results_table=None):
         """Run a query and return the result.
-
         If ``results_table`` is provided, then save the results
         into there (or just query from there if it already exists).
-
         Returns a ``google.cloud.bigquery.table.RowIterator``
         """
         if not results_table:
@@ -46,21 +49,17 @@ class BigQueryContext:
             full_res = self.client.query(
                 sql,
                 job_config=bigquery.QueryJobConfig(
-                    destination=self.client.dataset(
-                        self.dataset_id
-                    ).table(results_table)
-                )
+                    destination=self.client.dataset(self.dataset_id).table(
+                        results_table
+                    )
+                ),
             ).result()
-            print('Saved into', results_table)
+            print("Saved into", results_table)
             return full_res
 
         except Conflict:
             print("Full results table already exists. Reusing", results_table)
-            return self.client.query(
-                "SELECT * FROM {}".format(
-                    self.fully_qualify_table_name(results_table)
-                )
-            ).result()
+            return self.client.list_rows(results_table)
 
     def fully_qualify_table_name(self, table_name):
         """Given a table name, return it fully qualified."""
