@@ -85,6 +85,8 @@ class Experiment:
             factor '7' removes weekly seasonality, and the ``+1`` accounts
             for the fact that enrollment typically starts a few hours
             before UTC midnight.
+        app_id (str, optional): For a Glean app, the name of the BigQuery
+            dataset derived from its app ID, like `org_mozilla_firefox`.
 
     Attributes:
         experiment_slug (str): Name of the study, used to identify
@@ -103,12 +105,18 @@ class Experiment:
     experiment_slug = attr.ib()
     start_date = attr.ib()
     num_dates_enrollment = attr.ib(default=None)
+    app_id = attr.ib(default=None)
 
     def get_single_window_data(
-        self, bq_context, metric_list, last_date_full_data,
-        analysis_start_days, analysis_length_days,
-        enrollments_query_type='normandy', custom_enrollments_query=None,
-        segment_list=None
+        self,
+        bq_context,
+        metric_list,
+        last_date_full_data,
+        analysis_start_days,
+        analysis_length_days,
+        enrollments_query_type="normandy",
+        custom_enrollments_query=None,
+        segment_list=None,
     ):
         """Return a DataFrame containing per-client metric values.
 
@@ -169,30 +177,45 @@ class Experiment:
             standard format for queried experimental data.
         """
         time_limits = TimeLimits.for_single_analysis_window(
-            self.start_date, last_date_full_data, analysis_start_days,
-            analysis_length_days, self.num_dates_enrollment
+            self.start_date,
+            last_date_full_data,
+            analysis_start_days,
+            analysis_length_days,
+            self.num_dates_enrollment,
         )
 
-        sql_template = self.build_query(
-            metric_list=metric_list,
+        enrollments_sql = self.build_enrollments_query(
             time_limits=time_limits,
             enrollments_query_type=enrollments_query_type,
             custom_enrollments_query=custom_enrollments_query,
             segment_list=segment_list,
         )
 
-        full_res_table_name = sanitize_table_name_for_bq('_'.join(
-            [last_date_full_data, self.experiment_slug, hash_ish(sql_template)]
-        ))
+        enrollments_table = bq_context.run_query(enrollments_sql)
 
-        sql = sql_template.format(results_table=full_res_table_name)
+        metrics_sql = self.build_metrics_query(
+            metric_list=metric_list,
+            time_limits=time_limits,
+            enrollments_table=enrollments_table,
+        )
 
-        return bq_context.run_script_or_fetch(sql, full_res_table_name).to_dataframe()
+        full_res_table_name = sanitize_table_name_for_bq(
+            "_".join([last_date_full_data, self.experiment_slug, hash_ish(metrics_sql)])
+        )
+
+        return bq_context.run_query_and_fetch(
+            metrics_sql, full_res_table_name
+        ).to_dataframe()
 
     def get_time_series_data(
-        self, bq_context, metric_list, last_date_full_data,
-        time_series_period='weekly', enrollments_query_type='normandy',
-        custom_enrollments_query=None, segment_list=None
+        self,
+        bq_context,
+        metric_list,
+        last_date_full_data,
+        time_series_period="weekly",
+        enrollments_query_type="normandy",
+        custom_enrollments_query=None,
+        segment_list=None,
     ):
         """Return a TimeSeriesResult with per-client metric values.
 
@@ -247,45 +270,61 @@ class Experiment:
         """
 
         time_limits = TimeLimits.for_ts(
-            self.start_date, last_date_full_data, time_series_period,
-            self.num_dates_enrollment
+            self.start_date,
+            last_date_full_data,
+            time_series_period,
+            self.num_dates_enrollment,
         )
 
-        sql_template = self.build_query_template(
-            metric_list=metric_list,
+        enrollments_sql = self.build_enrollments_query(
             time_limits=time_limits,
             enrollments_query_type=enrollments_query_type,
             custom_enrollments_query=custom_enrollments_query,
             segment_list=segment_list,
         )
 
-        full_res_table_name = sanitize_table_name_for_bq('_'.join(
-            [last_date_full_data, self.experiment_slug, hash_ish(sql_template)]
-        ))
+        enrollments_table_name = sanitize_table_name_for_bq(
+            "_".join(
+                [
+                    last_date_full_data,
+                    "enrollments",
+                    self.experiment_slug,
+                    hash_ish(enrollments_sql),
+                ]
+            )
+        )
 
-        sql = sql_template.format(results_table=full_res_table_name)
+        enrollments_table = bq_context.run_query_and_fetch(
+            enrollments_sql, enrollments_table_name
+        )
 
-        bq_context.run_script_or_fetch(sql, full_res_table_name)
+        metrics_sql = self.build_metrics_query(
+            metric_list=metric_list,
+            time_limits=time_limits,
+            enrollments_table=enrollments_table,
+        )
+
+        full_res_table_name = sanitize_table_name_for_bq(
+            "_".join([last_date_full_data, self.experiment_slug, hash_ish(metrics_sql)])
+        )
+
+        bq_context.run_query_and_fetch(metrics_sql, full_res_table_name)
 
         return TimeSeriesResult(
             fully_qualified_table_name=bq_context.fully_qualify_table_name(
                 full_res_table_name
             ),
-            analysis_windows=time_limits.analysis_windows
+            analysis_windows=time_limits.analysis_windows,
         )
 
-    def build_query_template(
+    def build_enrollments_query(
         self,
-        metric_list,
         time_limits,
-        enrollments_query_type='normandy',
+        enrollments_query_type="normandy",
         custom_enrollments_query=None,
         segment_list=None,
     ) -> str:
-        """Return a SQL template for querying metric data.
-
-        The return value is a Python string template that needs to be formatted
-        with a value for `results_table` before it is valid SQL.
+        """Return a SQL query for querying enrollments data.
 
         For interactive use, prefer :meth:`.get_time_series_data` or
         :meth:`.get_single_window_data`, according to your use case,
@@ -293,11 +332,8 @@ class Experiment:
         dataframe.
 
         Args:
-            metric_list (list of mozanalysis.metric.Metric):
-                The metrics to analyze.
             time_limits (TimeLimits): An object describing the
                 interval(s) to query
-            results_table (str): The name of the table to write
             enrollments_query_type ('normandy' or 'fenix-fallback'):
                 Specifies the query type to use to get the experiment's
                 enrollments, unless overridden by
@@ -319,45 +355,69 @@ class Experiment:
             time_limits.analysis_windows
         )
 
-        enrollments_query = custom_enrollments_query or \
-            self._build_enrollments_query(time_limits, enrollments_query_type)
-
-        metrics_columns, metrics_joins = self._build_metrics_query_bits(
-            metric_list, time_limits
+        enrollments_query = custom_enrollments_query or self._build_enrollments_query(
+            time_limits, enrollments_query_type
         )
 
-        segments_query = self._build_segments_query(
-            segment_list, time_limits
-        )
+        segments_query = self._build_segments_query(segment_list, time_limits)
 
         return """
-    CREATE TEMPORARY TABLE enrollments AS (
-        WITH analysis_windows AS (
-            {analysis_windows_query}
-        ),
-        raw_enrollments AS ({enrollments_query}),
-        segmented_enrollments AS ({segments_query})
+            WITH analysis_windows AS (
+                {analysis_windows_query}
+            ),
+            raw_enrollments AS ({enrollments_query}),
+            segmented_enrollments AS ({segments_query})
 
-        SELECT
-            e.*,
-            aw.*
-        FROM segmented_enrollments e
-        CROSS JOIN analysis_windows aw
-    );
-
-    CREATE OR REPLACE TABLE {{results_table}} AS (
-        SELECT
-            enrollments.*,
-            {metrics_columns}
-        FROM enrollments
-        {metrics_joins}
-    );
+            SELECT
+                e.*,
+                aw.*
+            FROM segmented_enrollments e
+            CROSS JOIN analysis_windows aw
         """.format(
             analysis_windows_query=analysis_windows_query,
             enrollments_query=enrollments_query,
             segments_query=segments_query,
-            metrics_columns=',\n        '.join(metrics_columns),
-            metrics_joins='\n'.join(metrics_joins),
+        )
+
+    def build_metrics_query(
+        self,
+        metric_list,
+        time_limits,
+        enrollments_table,
+    ) -> str:
+        """Return a SQL query for querying metric data.
+
+        For interactive use, prefer :meth:`.get_time_series_data` or
+        :meth:`.get_single_window_data`, according to your use case,
+        which will run the query for you and return a materialized
+        dataframe.
+
+        Args:
+            metric_list (list of mozanalysis.metric.Metric):
+                The metrics to analyze.
+            time_limits (TimeLimits): An object describing the
+                interval(s) to query
+            enrollments_table (str): The name of the enrollments table
+
+        Returns:
+            A string containing a BigQuery SQL expression.
+
+        Building this query is the main goal of this module.
+        """
+        metrics_columns, metrics_joins = self._build_metrics_query_bits(
+            metric_list, time_limits
+        )
+
+        return """
+        SELECT
+            {enrollments_table}.*,
+            {metrics_columns}
+        FROM {enrollments_table}
+        {metrics_joins}
+        """.format(
+            metrics_columns=",\n        ".join(metrics_columns),
+            metrics_joins="\n".join(metrics_joins),
+            enrollments_table=enrollments_table,
         )
 
     @staticmethod
@@ -371,8 +431,7 @@ class Experiment:
         This method writes the SQL to define the analysis window table.
         """
         return "\n        UNION ALL\n        ".join(
-            "(SELECT {aws} AS analysis_window_start, {awe} AS analysis_window_end)"
-            .format(
+            "(SELECT {aws} AS analysis_window_start, {awe} AS analysis_window_end)".format(  # noqa:E501
                 aws=aw.start,
                 awe=aw.end,
             )
@@ -381,9 +440,9 @@ class Experiment:
 
     def _build_enrollments_query(self, time_limits, enrollments_query_type):
         """Return SQL to query a list of enrollments and their branches"""
-        if enrollments_query_type == 'normandy':
+        if enrollments_query_type == "normandy":
             return self._build_enrollments_query_normandy(time_limits)
-        elif enrollments_query_type == 'fenix-fallback':
+        elif enrollments_query_type == "fenix-fallback":
             return self._build_enrollments_query_fenix_baseline(time_limits)
         else:
             raise ValueError
@@ -432,7 +491,7 @@ class Experiment:
                 '{experiment_slug}'
             ).branch,
             DATE(MIN(b.submission_timestamp)) AS enrollment_date
-        FROM `moz-fx-data-shared-prod.org_mozilla_fenix.baseline` b
+        FROM `moz-fx-data-shared-prod.{dataset}.baseline` b
         WHERE
             DATE(b.submission_timestamp)
                 BETWEEN DATE_SUB('{first_enrollment_date}', INTERVAL 7 DAY)
@@ -447,6 +506,7 @@ class Experiment:
             experiment_slug=self.experiment_slug,
             first_enrollment_date=time_limits.first_enrollment_date,
             last_enrollment_date=time_limits.last_enrollment_date,
+            dataset=self.app_id or "org_mozilla_firefox",
         )
 
     def _build_metrics_query_bits(self, metric_list, time_limits):
@@ -462,22 +522,21 @@ class Experiment:
 
         for i, ds in enumerate(ds_metrics.keys()):
             query_for_metrics = ds.build_query(
-                ds_metrics[ds], time_limits, self.experiment_slug
+                ds_metrics[ds], time_limits, self.experiment_slug, self.app_id
             )
             metrics_joins.append(
                 """    LEFT JOIN (
         {query}
         ) ds_{i} USING (client_id, analysis_window_start, analysis_window_end)
                 """.format(
-                    query=query_for_metrics,
-                    i=i
+                    query=query_for_metrics, i=i
                 )
             )
 
             for m in ds_metrics[ds]:
-                metrics_columns.append("ds_{i}.{metric_name}".format(
-                    i=i, metric_name=m.name
-                ))
+                metrics_columns.append(
+                    "ds_{i}.{metric_name}".format(i=i, metric_name=m.name)
+                )
 
         return metrics_columns, metrics_joins
 
@@ -514,8 +573,8 @@ class Experiment:
         FROM raw_enrollments
         {segments_joins}
         """.format(
-            segments_columns=',\n        '.join(segments_columns),
-            segments_joins='\n'.join(segments_joins)
+            segments_columns=",\n        ".join(segments_columns),
+            segments_joins="\n".join(segments_joins),
         )
 
     def _build_segments_query_bits(self, segment_list, time_limits):
@@ -527,22 +586,21 @@ class Experiment:
 
         for i, ds in enumerate(ds_segments.keys()):
             query_for_segments = ds.build_query(
-                ds_segments[ds], time_limits, self.experiment_slug
+                ds_segments[ds], time_limits, self.experiment_slug, self.app_id
             )
             segments_joins.append(
                 """    LEFT JOIN (
         {query}
         ) ds_{i} USING (client_id)
                 """.format(
-                    query=query_for_segments,
-                    i=i
+                    query=query_for_segments, i=i
                 )
             )
 
             for m in ds_segments[ds]:
-                segments_columns.append("ds_{i}.{segment_name}".format(
-                    i=i, segment_name=m.name
-                ))
+                segments_columns.append(
+                    "ds_{i}.{segment_name}".format(i=i, segment_name=m.name)
+                )
 
         return segments_columns, segments_joins
 
@@ -643,9 +701,11 @@ class TimeLimits:
             raise ValueError(
                 "You said you wanted {} dates of enrollment, ".format(
                     num_dates_enrollment
-                ) + "and need data from the {}th day after enrollment. ".format(
+                )
+                + "and need data from the {}th day after enrollment. ".format(
                     analysis_window.end
-                ) + "For that, you need to wait until we have data for {}.".format(
+                )
+                + "For that, you need to wait until we have data for {}.".format(
                     last_date_data_required
                 )
             )
@@ -681,32 +741,32 @@ class TimeLimits:
                 enrollments. This is a mandatory argument because it
                 determines the number of points in the time series.
         """
-        if time_series_period not in ('daily', 'weekly'):
-            raise ValueError("Unsupported time series period {}".format(
-                time_series_period
-            ))
+        if time_series_period not in ("daily", "weekly"):
+            raise ValueError(
+                "Unsupported time series period {}".format(time_series_period)
+            )
 
         if num_dates_enrollment <= 0:
             raise ValueError("Number of enrollment dates must be a positive number")
 
-        analysis_window_length_dates = 1 if time_series_period == 'daily' else 7
+        analysis_window_length_dates = 1 if time_series_period == "daily" else 7
 
-        last_enrollment_date = add_days(
-            first_enrollment_date, num_dates_enrollment - 1
-        )
+        last_enrollment_date = add_days(first_enrollment_date, num_dates_enrollment - 1)
         max_dates_of_data = date_sub(last_date_full_data, last_enrollment_date) + 1
         num_periods = max_dates_of_data // analysis_window_length_dates
 
         if num_periods <= 0:
             raise ValueError("Insufficient data")
 
-        analysis_windows = tuple([
-            AnalysisWindow(
-                i * analysis_window_length_dates,
-                (i + 1) * analysis_window_length_dates - 1
-            )
-            for i in range(num_periods)
-        ])
+        analysis_windows = tuple(
+            [
+                AnalysisWindow(
+                    i * analysis_window_length_dates,
+                    (i + 1) * analysis_window_length_dates - 1,
+                )
+                for i in range(num_periods)
+            ]
+        )
 
         last_date_data_required = add_days(
             last_enrollment_date, analysis_windows[-1].end
@@ -761,6 +821,7 @@ class AnalysisWindow:
         end (int): Final day of the analysis window, in days since
             enrollment.
     """
+
     start = attr.ib(type=int)
     end = attr.ib(type=int)
 
@@ -794,6 +855,7 @@ class TimeSeriesResult:
 
         window_0 = time_series_result.get(bq_context, 0)
     """
+
     fully_qualified_table_name = attr.ib(type=str)
     analysis_windows = attr.ib(type=list)
 
@@ -818,7 +880,7 @@ class TimeSeriesResult:
                     "AnalysisWindow not found with start of {}".format(analysis_window)
                 )
 
-        return bq_context.run_query(
+        return bq_context.run_query_and_fetch(
             self._build_analysis_window_subset_query(analysis_window)
         ).to_dataframe()
 
